import sys
from pathlib import Path

import numpy as np
import xarray as xr


# to be modified (will update the given attrs)
run_attrs = {
    'activity_id': 'HighResMIP',
    'experiment': "coupled control with fixed 1950's forcing (HighResMIP equivalent of pre-industrial control)",
    'experiment_id': "control-1950",
    'further_info_url': "https://furtherinfo.es-doc.org/CMIP6.EC-Earth-Consortium.EC-Earth3P-VHR.control-1950.none.r1i1p1f1",
    'physics_index': "1",
    'variant_label': "r1i1p1f1",
}

run_attrs = {
     'nominal_resolution': "10 km"
}

# maybe more variables to be added (if given will replace the var attrs)
var_attrs = {
    'psl': {
        "standard_name": "air_pressure_at_mean_sea_level",
        "units": "Pa",
        "cell_methods": "area: time: mean",
        "cell_measures": "area: areacella",
        "long_name": "Sea Level Pressure",
        "comment": "Sea Level Pressure"
    },
    'pr': {
        "standard_name": "precipitation_flux",
        "units": "kg m-2 s-1",
        "cell_methods": "area: time: mean",
        "cell_measures": "area: areacella",
        "long_name": "Precipitation",
        "comment": "includes both liquid and solid phases"
    }
}


def clean_cmor(file, lat, lon, lat_bnds, lon_bnds):
    save_path = Path(file)
    save_path = save_path.parent.parent / save_path.name
    if save_path.exists():
        return
        raise FileExistsError(save_path)

    print("Loading file: %s" % file)
    ds = xr.open_dataset(file)

    # set new lon, lat and bounds in the file
    da = ds.drop_vars(("vertices_longitude", "vertices_latitude"))
    var = da.attrs['variable_id']
    da[var] = da[var].astype("float32", copy=False)
    da['lat'] = lat
    da['lon'] = lon
    da['lat_bnds'] = lat_bnds
    da['lon_bnds'] = lon_bnds
    da = da.swap_dims({"i": "lon", "j": "lat"}).drop_vars(
        ("i", "j", "longitude", "latitude"))

    # Uniformize some other attrs
    da.attrs.update(run_attrs)

    if var in var_attrs:
        da[var].attrs = var_attrs[var]

    # set encoding to do not include coordinates
    da[var].encoding["coordinates"] = None
    da.lat_bnds.encoding["coordinates"] = None
    da.lon_bnds.encoding["coordinates"] = None
    da.time.encoding["units"] = "days since 1950-01-01"
    da.time.encoding["dtype"] = "double"
    da.time.encoding["_FillValue"] = None
    da.time_bnds.encoding["dtype"] = "double"
    da.time_bnds.encoding["_FillValue"] = None

    # set encoding for all the dimensions and vars
    def_enconding = {"_FillValue": None, "dtype": "float32"}
    encoding = {ivar: def_enconding for ivar in da.data_vars}
    encoding.update({ivar: def_enconding for ivar in da.coords})
    del encoding["time"]
    del encoding["time_bnds"]
    encoding[var] = {
        "missing_value": 1e20,
        "_FillValue": 1e20
    }

    # save
    da.to_netcdf(save_path, unlimited_dims='time', encoding=encoding)
    ds.close()
    print("File saved: %s" % save_path)


def main(files):
    ds = xr.open_dataset(files[-1])
    # get 1D lon lat and their bounds
    lat = ds.latitude[:, 0].astype("float32", copy=False)
    lon = ds.longitude[0].astype("float32", copy=False)
    lat_bnds = ds.vertices_latitude[:, 0, ::2].rename({"vertices": "bnds"}).astype("float32", copy=False)
    lon_bnds = ds.vertices_longitude[0, :, :2].rename({"vertices": "bnds"}).astype("float32", copy=False)
    ds.close()

    # check if coordinates and their bounds are ordered
    check_coords(lat, lat_bnds)
    check_coords(lon, lon_bnds)

    # update their attributes
    lat.attrs.update({
        'bounds': 'lat_bnds',
        'units': 'degrees_north',
        'axis': 'Y',
        'long_name': 'Latitude',
        'standard_name': 'latitude'
    })
    lon.attrs.update({
        'bounds': 'lon_bnds',
        'units': 'degrees_east',
        'axis': 'X',
        'long_name': 'Longitude',
        'standard_name': 'longitude'
    })
    lat_bnds.attrs = {}
    lon_bnds.attrs = {}

    for file in files:
        clean_cmor(file, lat, lon, lat_bnds, lon_bnds)


def check_coords(coord, bnds):
    def check_coord():
        """Check if coordinate is ordered"""
        return np.all(np.diff(coord) > 0)

    def check_bnds():
        """Check if coordinate bounds are ordered"""
        return np.all(coord > bnds[:, 0]) and np.all(coord < bnds[:, 1])

    if not check_coord():
        print("Reordering coordinate:\n%s" % coord)
        coord.values = coord.values[::-1]
        assert check_coord(), "Cannot order the coordinate:\n%s" % coord

    if not check_bnds():
        print("Reordering coordinate bounds:\n%s" % bnds)
        bnds_values = bnds.values
        if bnds_values[0][0] > bnds_values[0][1]:
            bnds_values = bnds_values[:, ::-1]
        if bnds_values[0][0] > bnds_values[1][0]:
            bnds_values = bnds_values[::-1]
        bnds.values = bnds_values
        assert check_bnds(), "Cannot order the coordinate bounds:\n%s" % bnds

if __name__ == '__main__':
    main(sys.argv[1:])

